Section

Enum Section 

pub enum Section {
    VectorTable,
    Text,
    Rodata,
    Bss,
    Data,
    Uninit,
}
Represents one of the memory regions in the linker script

Variants



VectorTable

The .vector_table section

Contains the reset and exception vectors



Text

The .text section

Contains the executable code



Rodata

The .rodata section

Contains read-only static data



Bss

The .bss section

Contains zero-initialised static data



Data

The .data section

Contains non-zero-initialised static data



Uninit

The .uninit section

Contains non-initialised static data

Implementations

Source

impl Section

pub fn iter() -> impl Iterator<Item = Section>

Create an iterator over all the regions

pub fn top(&self) -> *const u8

Get the highest address of this region

Technically it gets one past the end of the region.

pub fn bottom(&self) -> *const u8

Get the lowest address of this region

pub fn range(&self) -> Option<Range<*const u8>>

Get the range of this region.

pub fn mpu_range(&self) -> Option<RangeInclusive<*const u8>>

Get the inclusive range of this region.

This is the range you need to give to the PMSAv8 MPU code.