Stack

Enum Stack 

pub enum Stack {
    Und,
    Svc,
    Abt,
    Hyp,
    Irq,
    Fiq,
    Sys,
}
Represents one of the AArch32 stacks

Variants



Und

UND mode stack



Svc

SVC mode stack



Abt

ABT mode stack, for data abort and prefetch abort



Hyp

HYP mode stack, for EL2



Irq

IRQ mode stack, for interrupts



Fiq

FIQ mode stack, for fast interrupts



Sys

SYS mode stack, for the main thread

Implementations

Source

impl Stack

pub fn iter() -> impl Iterator<Item = Stack>

Create an iterator over all the stacks

pub fn top(&self, core: usize) -> Option<*const u32>

Get the highest address of this stack, for the given core

pub fn bottom(&self, core: usize) -> Option<*const u32>

Get the lowest address of this stack, for the given core

pub fn range(&self, core: usize) -> Option<Range<*const u32>>

Get the range of this stack, for the given core

pub fn mpu_range(&self, core: usize) -> Option<RangeInclusive<*const u8>>

Get the inclusive range of this stack, for the given core

This is the range you need to give to the PMSAv8 MPU code.

pub fn num_cores() -> usize

Get number of cores in this system

pub fn per_core_size_words(&self) -> isize

Get the total size of this stack across all cores