#[repr(C)]pub struct Frame {
pub r0: u32,
pub r1: u32,
pub r2: u32,
pub r3: u32,
pub r4: u32,
pub r5: u32,
}Expand description
Arguments stacked on interrupt
This struct is very carefully designed to match the layout of the registers pushed to the stack in our SVC handler.
Fields
r0: u32r1: u32r2: u32r3: u32r4: u32r5: u32Trait Implementations
impl Eq for Frame
impl StructuralPartialEq for Frame