Code to get the memory regions from the linker script
This is useful if you want to set up the MPU
Structs
- Section
Iter - Iterator over all the [
Region] variants
Enums
- Section
- Represents one of the memory regions in the linker script